![](../../workflows/test%20build/badge.svg)

# muni-chat-service
Example of a web service with OpenAPI description implemented using Spring Boot.

This project demonstrates the following features:
* a multimodule Maven project
  * parent project [chat-service-parent](pom.xml) and multiple modules:
    * module [chat-server](chat-server/pom.xml) - executable server
    * module [chat-client-java-lib](chat-client-java-lib/pom.xml) - Java client library generated by OpenAPI Generator "java"
    * module [chat-client-java](chat-client-java/pom.xml) - executable client using the library
    * module [chat-server-generated](chat-server-generated/pom.xml) - executable server generated by OpenAPI Generator "spring"
  * module aggregation, where a command executed in the parent project gets executed also in all modules
  * module inheritance, where the parent project inherits from `org.springframework.boot:spring-boot-starter-parent`
    and each module inherits from the parent
* a Spring Boot application ([ChatApplication.java](chat-server/src/main/java/cz/muni/chat/server/ChatApplication.java))
* a Spring MVC's REST controller ([ChatRestController.java](chat-server/src/main/java/cz/muni/chat/server/rest/ChatRestController.java))
* application with multiple layers
  * [facade](chat-server/src/main/java/cz/muni/chat/server/facade/) - a stable API for the outside world
  * [service layer](chat-server/src/main/java/cz/muni/chat/server/service/) - internal business logic
  * persistence layer is omitted for the sake of simplicity
* OpenAPI description of the app's API ([openapi.yaml](openapi.yaml)) generated from annotated Java code in the REST
  controller and facade classes
* unit tests executed when the command `mvn test` is issued ([ChatRestControllerUnitTests.java](chat-server/src/test/java/cz/muni/chat/server/rest/ChatRestControllerUnitTests.java))
* integration tests executed when the command `mvn verify` is issued ([ChatIT.java](chat-server/src/test/java/cz/muni/chat/server/rest/ChatIT.java))

## Installation

Prerequisites: git, [Apache Maven](https://maven.apache.org/) and JDK 17+ 

Download and compile:
```bash
git clone https://github.com/martin-kuba/muni-chat-service.git
cd muni-chat-service
mvn install
```

## Run server

```bash
cd chat-server
mvn spring-boot:run
```
Then visit the service with your browser: http://localhost:8080/


## Run client
```bash
cd chat-client-java
mvn spring-boot:run
```

## Advanced usage

### Running the server with TLS enabled

Create a PKCS12 keystore with:
```bash
openssl pkcs12 -export -name "mycert" -inkey key.pem -in cert.pem -certfile chain.pem -out mykeystore.p12
```
then run with the following options:
```bash
java -jar target/chat_service.jar \
     --server.port=8443 \
     --server.ssl.key-store-type=pkcs12 \
     --server.ssl.key-store=mykeystore.p12 \
     --server.ssl.key-store-password=password
```

### Running the server behind Apache web server

```bash
java -jar target/chat_service.jar \
     --server.tomcat.ajp.port=8009 \
     --server.tomcat.ajp.enabled=true \
     --server.tomcat.ajp.secret=5ecr3t
```